/*
	desc      : JSON library wrapper (WebOs cjson)
	author    : raphael smart
	date      : may 2012

	licence/disclaimer:

	This software is provided 'as-is', without any express or implied warranty.
	In no event will the author be held liable for any damages arising from
	the use of this software.
*/

namespace libcjson
constant __MODULENAME__ = "libcjson"

include std/machine.e
include std/dll.e

include lib/clib.e

/* imports */

constant libcjson_lib = link_dll("libcjson","./lib/")

constant _json_tokener_new_              = define_c_func(libcjson_lib, "json_tokener_new", {}, C_POINTER)
constant _json_tokener_free_             = define_c_proc(libcjson_lib, "json_tokener_free", {C_POINTER})
constant _json_tokener_reset_            = define_c_proc(libcjson_lib, "json_tokener_reset", {C_POINTER})
constant _json_tokener_parse_            = define_c_func(libcjson_lib, "json_tokener_parse", {C_POINTER}, C_POINTER)
constant _json_tokener_parse_ex_         = define_c_func(libcjson_lib, "json_tokener_parse_ex", {C_POINTER, C_POINTER, C_INT}, C_POINTER)
constant _json_object_get_               = define_c_func(libcjson_lib, "json_object_get", {C_POINTER}, C_POINTER)
constant _json_object_put_               = define_c_proc(libcjson_lib, "json_object_put", {C_POINTER})
constant _json_object_is_type_           = define_c_func(libcjson_lib, "json_object_is_type", {C_POINTER, C_INT}, C_INT)
constant _json_object_get_type_          = define_c_func(libcjson_lib, "json_object_get_type", {C_POINTER}, C_INT)
constant _json_object_to_json_string_    = define_c_func(libcjson_lib, "json_object_to_json_string", {C_POINTER}, C_POINTER)
constant _json_object_new_object_        = define_c_func(libcjson_lib, "json_object_new_object", {}, C_POINTER)
constant _json_object_get_object_        = define_c_func(libcjson_lib, "json_object_get_object", {C_POINTER}, C_POINTER)
constant _json_object_object_add_        = define_c_proc(libcjson_lib, "json_object_object_add", {C_POINTER, C_POINTER, C_POINTER})
constant _json_object_object_get_        = define_c_func(libcjson_lib, "json_object_object_get", {C_POINTER, C_POINTER}, C_POINTER)
constant _json_object_object_get_ex_     = define_c_func(libcjson_lib, "json_object_object_get_ex", {C_POINTER, C_POINTER, C_POINTER}, C_SHORT)
constant _json_object_object_del_        = define_c_proc(libcjson_lib, "json_object_object_del", {C_POINTER, C_POINTER})
constant _json_object_new_array_         = define_c_func(libcjson_lib, "json_object_new_array", {}, C_POINTER)
constant _json_object_get_array_         = define_c_func(libcjson_lib, "json_object_get_array", {C_POINTER}, C_POINTER)
constant _json_object_array_length_      = define_c_func(libcjson_lib, "json_object_array_length", {C_POINTER}, C_INT)
constant _json_object_array_add_         = define_c_func(libcjson_lib, "json_object_array_add", {C_POINTER, C_POINTER}, C_INT)
constant _json_object_array_put_idx_     = define_c_func(libcjson_lib, "json_object_array_put_idx", {C_POINTER, C_INT, C_POINTER}, C_INT)
constant _json_object_array_get_idx_     = define_c_func(libcjson_lib, "json_object_array_get_idx", {C_POINTER, C_INT}, C_POINTER)
constant _json_object_new_boolean_       = define_c_func(libcjson_lib, "json_object_new_boolean", {C_SHORT}, C_POINTER)
constant _json_object_get_boolean_       = define_c_func(libcjson_lib, "json_object_get_boolean", {C_POINTER}, C_SHORT)
constant _json_object_new_int_           = define_c_func(libcjson_lib, "json_object_new_int", {C_INT}, C_POINTER)
constant _json_object_get_int_           = define_c_func(libcjson_lib, "json_object_get_int", {C_POINTER}, C_INT)
constant _json_object_new_double_        = define_c_func(libcjson_lib, "json_object_new_double", {C_DOUBLE}, C_POINTER)
constant _json_object_get_double_        = define_c_func(libcjson_lib, "json_object_get_double", {C_POINTER}, C_DOUBLE)
constant _json_object_new_string_        = define_c_func(libcjson_lib, "json_object_new_string", {C_POINTER}, C_POINTER)
constant _json_object_new_string_len_    = define_c_func(libcjson_lib, "json_object_new_string_len", {C_POINTER, C_INT}, C_POINTER)
constant _json_object_get_string_        = define_c_func(libcjson_lib, "json_object_get_string", {C_POINTER}, C_POINTER)
constant _json_object_from_file_         = define_c_func(libcjson_lib, "json_object_from_file", {C_POINTER}, C_POINTER)
constant _json_object_to_file_           = define_c_func(libcjson_lib, "json_object_to_file", {C_POINTER, C_POINTER}, C_INT)
constant _json_object_iter_init_default_ = define_c_func(libcjson_lib, "json_object_iter_init_default", {}, C_POINTER)
constant _json_object_iter_begin_        = define_c_func(libcjson_lib, "json_object_iter_begin", {C_POINTER}, C_POINTER)
constant _json_object_iter_end_          = define_c_func(libcjson_lib, "json_object_iter_end", {C_POINTER}, C_POINTER)
constant _json_object_iter_next_         = define_c_proc(libcjson_lib, "json_object_iter_next", {C_POINTER})
constant _json_object_iter_peek_name_    = define_c_func(libcjson_lib, "json_object_iter_peek_name", {C_POINTER}, C_POINTER)
constant _json_object_iter_peek_value_   = define_c_func(libcjson_lib, "json_object_iter_peek_value", {C_POINTER}, C_POINTER)
constant _json_object_iter_equal_        = define_c_func(libcjson_lib, "json_object_iter_equal", {C_POINTER, C_POINTER}, C_SHORT)

enum 
	false  = 0,
	true

/* #define */

constant JSON_FILE_BUF_SIZE = 4096

/* enum */

public enum

  json_type_null = 0,
  json_type_boolean,
  json_type_double,
  json_type_int,
  json_type_object,
  json_type_array,
  json_type_string

/* struct */

memstruct lh_entry 
	pointer int k    --// void *k;               /*** The key. */
	pointer int v    --// void *v;               /*** The value. */
	pointer int next --// struct lh_entry *next; /*** The next entry */
	pointer int prev --// struct lh_entry *prev; /*** The previous entry. */
end memstruct

memstruct lh_table
	int size              /*** Size of our hash. */
	int count             /*** Numbers of entries. */
	int collisions        /*** Number of collisions. */
	int resizes           /*** Number of resizes. */
	int lookups           /*** Number of lookups. */
	int inserts           /*** Number of inserts. */
	int deletes           /*** Number of deletes. */
	pointer char name     /*** Name of the hash table. */
	pointer int head      --// struct lh_entry *head /*** The first entry. */
	pointer int tail      --// struct lh_entry *tail /*** The last entry.  */
	pointer int table     --// struct lh_entry *table
	/**
	* A pointer onto the function responsible for freeing an entry.
	*/
	pointer int free_fn   --// lh_entry_free_fn *free_fn
	pointer int hash_fn   --// lh_hash_fn *hash_fn
	pointer int equal_fn  --// lh_equal_fn *equal_fn
end memstruct

public memstruct object_iterator
	pointer int _opaque
end memstruct

/* exports */

public function tokener_new()
    return c_func(_json_tokener_new_, {})
end function

public procedure tokener_free(atom tok)
    c_proc(_json_tokener_free_, {tok})
end procedure

public procedure tokener_reset(atom tok)
    c_proc(_json_tokener_reset_, {tok})
end procedure

public function tokener_parse(sequence seq_str)
    object result
    atom str
	 str = allocate_string(seq_str)
    result = c_func(_json_tokener_parse_, {str})
    free(str)
    return result
end function

public function tokener_parse_ex(atom tok, sequence seq_str, integer len)
    object result
    atom str
    str = allocate_string(seq_str)
    result = c_func(_json_tokener_parse_ex_, {tok, str, len})
    free(str)
    return result
end function

public function get(atom obj)
    return c_func(_json_object_get_, {obj})
end function

public procedure put(atom obj)
    c_proc(_json_object_put_, {obj})
end procedure

public function is_type(atom obj, integer _type)
    return c_func(_json_object_is_type_, {obj, _type})
end function

public function get_type(atom obj)
    return c_func(_json_object_get_type_, {obj})
end function

public function to_json_string(atom obj)
    object result = c_func(_json_object_to_json_string_, {obj})
    return peek_string( result )
end function

public function new_object()
    return c_func(_json_object_new_object_, {})
end function

public function get_object(atom obj)
    return c_func(_json_object_get_object_, {obj})
end function

public procedure object_add(atom obj, sequence seq_key, atom val)
    atom key
    key = allocate_string(seq_key)
    c_proc(_json_object_object_add_, {obj, key, val})
	 free(key)
end procedure

public function object_get(atom obj, sequence seq_key)
    object result
    atom key
    key = allocate_string(seq_key)
    result = c_func(_json_object_object_get_, {obj, key})
    free(key)
    return result
end function

public function object_get_ex(atom obj, sequence seq_key, atom value)
    object result
    atom key
    key = allocate_string(seq_key)
    result = c_func(_json_object_object_get_ex_, {obj, key, value})
    free(key)
    return result
end function

public procedure object_del(atom obj, sequence seq_key)
    atom key
    key = allocate_string(seq_key)
    c_proc(_json_object_object_del_, {obj, key})
	 free(key)
end procedure

public function new_array()
    return c_func(_json_object_new_array_, {})
end function

public function get_array(atom obj)
    return c_func(_json_object_get_array_, {obj})
end function

public function array_length(atom obj)
    return c_func(_json_object_array_length_, {obj})
end function

public function array_add(atom obj, atom val)
    return c_func(_json_object_array_add_, {obj, val})
end function

public function array_put_idx(atom obj, integer idx, atom val)
    return c_func(_json_object_array_put_idx_, {obj, idx, val})
end function

public function array_get_idx(atom obj, integer idx)
    return c_func(_json_object_array_get_idx_, {obj, idx})
end function

public function new_boolean(integer b)
    return c_func(_json_object_new_boolean_, {b})
end function

public function get_boolean(atom obj)
    return c_func(_json_object_get_boolean_, {obj})
end function

public function new_int(integer i)
    return c_func(_json_object_new_int_, {i})
end function

public function get_int(atom obj)
    return c_func(_json_object_get_int_, {obj})
end function

public function new_double(atom d)
    return c_func(_json_object_new_double_, {d})
end function

public function get_double(atom obj)
    return c_func(_json_object_get_double_, {obj})
end function

public function new_string(sequence seq_s)
    object result
    atom s
	 s = allocate_string(seq_s)
    result = c_func(_json_object_new_string_, {s})
    free(s)
    return result
end function

public function new_string_len(sequence seq_s, integer len)
    object result
    atom s
    s = allocate_string(seq_s)
    result = c_func(_json_object_new_string_len_, {s, len})
    free(s)
    return result
end function

public function get_string(atom obj)
    object result = c_func(_json_object_get_string_, {obj})
    return peek_string( result )
end function

public function from_file(sequence seq_filename)
    object result
    atom filename
    filename = allocate_string(seq_filename)
    result = c_func(_json_object_from_file_, {filename})
    free(filename)
    return result
end function

public function to_file(sequence seq_filename, atom obj)
    object result
    atom filename
    filename = allocate_string(seq_filename)
    result = c_func(_json_object_to_file_, {filename, obj})
    free(filename)
    return result
end function

public function iter_init_default()
    object result
    result = c_func(_json_object_iter_init_default_, {})
    return result
end function

public function iter_begin(atom obj)
    object result
    result = c_func(_json_object_iter_begin_, {obj})
    return result
end function

public function iter_end(atom obj)
    object result
    result = c_func(_json_object_iter_end_, {obj})
    return result
end function

public procedure iter_next(atom iter)
    c_proc(_json_object_iter_next_, {iter})
end procedure

public function iter_peek_name(atom iter)
    object result = c_func(_json_object_iter_peek_name_, {iter})
    return peek_string( result )
end function

public function iter_peek_value(atom iter)
    return c_func(_json_object_iter_peek_value_, {iter})
end function

public function iter_equal(atom iter1, atom iter2)
    return c_func(_json_object_iter_equal_, {iter1, iter2})
end function

public procedure foreach(atom obj, atom rid)
	if (obj > 0) and (rid > 0) then
		/* 
			iterate through object list 
			extract key
			extract value
			call user defined function

			#define json_object_object_foreach(obj,key,val) \
			char *key; struct json_object *val; \
			for (struct lh_entry *entry = json_object_get_object(obj)->head; ({ if(entry) { key = (char*)entry->k; val = (struct json_object*)entry->v; } ; entry; }); entry = entry->next )
		*/
		atom _entry = get_object(obj)
		_entry      = _entry.lh_table.head
		if _entry then
			while _entry != #0 do
				call_proc(rid, { peek_string(_entry.lh_entry.k), _entry.lh_entry.v })
				_entry = _entry.lh_entry.next
			end while
		end if
	end if
end procedure

public function is_error(atom ptr)
	return ptr < 0
end function

ifdef __TEST_CJSON_IMPORTS__ then
	printf(1, "%-40s = %s\n", { "__MODULENAME__", __MODULENAME__})
	printf(1, "%-40s = %d\n", { "libcjson_lib", libcjson_lib})
	printf(1, "%-40s = %d\n", { "_json_tokener_new_", _json_tokener_new_})
	printf(1, "%-40s = %d\n", { "_json_tokener_free_", _json_tokener_free_})
	printf(1, "%-40s = %d\n", { "_json_tokener_reset_", _json_tokener_reset_})
	printf(1, "%-40s = %d\n", { "_json_tokener_parse_", _json_tokener_parse_})
	printf(1, "%-40s = %d\n", { "_json_tokener_parse_ex_", _json_tokener_parse_ex_})
	printf(1, "%-40s = %d\n", { "_json_object_get_", _json_object_get_})
	printf(1, "%-40s = %d\n", { "_json_object_put_", _json_object_put_})
	printf(1, "%-40s = %d\n", { "_json_object_is_type_", _json_object_is_type_})
	printf(1, "%-40s = %d\n", { "_json_object_get_type_", _json_object_get_type_})
	printf(1, "%-40s = %d\n", { "_json_object_to_json_string_", _json_object_to_json_string_})
	printf(1, "%-40s = %d\n", { "_json_object_new_object_", _json_object_new_object_})
	printf(1, "%-40s = %d\n", { "_json_object_get_object_", _json_object_get_object_})
	printf(1, "%-40s = %d\n", { "_json_object_object_add_", _json_object_object_add_})
	printf(1, "%-40s = %d\n", { "_json_object_object_get_", _json_object_object_get_})
	printf(1, "%-40s = %d\n", { "_json_object_object_get_ex_", _json_object_object_get_ex_})
	printf(1, "%-40s = %d\n", { "_json_object_object_del_", _json_object_object_del_})
	printf(1, "%-40s = %d\n", { "_json_object_new_array_", _json_object_new_array_})
	printf(1, "%-40s = %d\n", { "_json_object_get_array_", _json_object_get_array_})
	printf(1, "%-40s = %d\n", { "_json_object_array_length_", _json_object_array_length_})
	printf(1, "%-40s = %d\n", { "_json_object_array_add_", _json_object_array_add_})
	printf(1, "%-40s = %d\n", { "_json_object_array_put_idx_", _json_object_array_put_idx_})
	printf(1, "%-40s = %d\n", { "_json_object_array_get_idx_", _json_object_array_get_idx_})
	printf(1, "%-40s = %d\n", { "_json_object_new_boolean_", _json_object_new_boolean_})
	printf(1, "%-40s = %d\n", { "_json_object_get_boolean_", _json_object_get_boolean_})
	printf(1, "%-40s = %d\n", { "_json_object_new_int_", _json_object_new_int_})
	printf(1, "%-40s = %d\n", { "_json_object_get_int_", _json_object_get_int_})
	printf(1, "%-40s = %d\n", { "_json_object_new_double_", _json_object_new_double_})
	printf(1, "%-40s = %d\n", { "_json_object_get_double_", _json_object_get_double_})
	printf(1, "%-40s = %d\n", { "_json_object_new_string_", _json_object_new_string_})
	printf(1, "%-40s = %d\n", { "_json_object_new_string_len_", _json_object_new_string_len_})
	printf(1, "%-40s = %d\n", { "_json_object_get_string_", _json_object_get_string_})
	printf(1, "%-40s = %d\n", { "_json_object_from_file_", _json_object_from_file_})
	printf(1, "%-40s = %d\n", { "_json_object_to_file_", _json_object_to_file_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_init_default_", _json_object_iter_init_default_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_begin_", _json_object_iter_begin_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_end_", _json_object_iter_end_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_next_", _json_object_iter_next_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_peek_name_", _json_object_iter_peek_name_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_peek_value_", _json_object_iter_peek_value_})
	printf(1, "%-40s = %d\n", { "_json_object_iter_equal_", _json_object_iter_equal_})
end ifdef
