#!/bin/sh

OS=$(uname)
MODULE=cjson
LIBNAME="lib$MODULE"
LIBEXT=dll
CFLAGS="-DHAVE_CONFIG_H -I. -Wall -std=gnu99 -D_GNU_SOURCE -D_REENTRANT -Werror -O2"
cp config/config.mingw.h config.h

if [ "$OS" = "Linux" ]; then
	LIBEXT=so
	CFLAGS="-fPIC -DPIC"
	cp config/config.h .
fi

compile()
{
	SRC=$1
	printf "$OS : compiling $SRC ..."
	gcc $CFLAGS -MT $SRC.lo -MD -MP -MF deps/$SRC.Tpo -c $SRC.c -o libs/$SRC.o && \
	mv -f deps/$SRC.Tpo deps/$SRC.Plo
	printf " done\n"
}

compile_exe()
{
	SRC=$1
	compile $SRC
	gcc -o $SRC ./libs/$SRC.o -L ./libs -lcjson
}

linklib()
{
	echo "$OS : linking $LIBNAME.$LIBEXT"
	gcc -shared  libs/arraylist.o libs/debug.o libs/json_object.o libs/json_object_iterator.o libs/json_tokener.o libs/json_util.o libs/linkhash.o libs/printbuf.o -Wl,-soname -Wl,$LIBNAME.$LIBEXT -o libs/$LIBNAME.$LIBEXT
	ar cru libs/$LIBNAME.a  libs/arraylist.o libs/debug.o libs/json_object.o libs/json_object_iterator.o libs/json_tokener.o libs/json_util.o libs/linkhash.o libs/printbuf.o
	ranlib libs/$LIBNAME.a
}

mkdir deps 2> /dev/null
mkdir libs 2> /dev/null

# shared library
compile arraylist
compile debug
compile json_object
compile json_object_iterator
compile json_tokener
compile json_util
compile linkhash
compile printbuf

linklib

# test
compile_exe test1
